@@ -6,21 +6,19 @@
 #include <string>
 #include <list>
 
+/*------- class declaration:
+-------------------------------------------------------------------*/
 class BtStringTool
 {
-public:
-	BtStringTool() {}
-	virtual ~BtStringTool() {}
-
 //******* CONSTANTS *******
 private:
-	static const std::string WhiteSpaces;
+   static const std::string WhiteSpaces;
 
 //******* METHODS *******
 public:
-	bool         is_white_space ( const char );
-	bool         is_white_string( const std::string& );
-	void         split          ( const std::string&, std::list<std::string>&, const char = ' ' );
+   bool         is_white_space ( const char );
+   bool         is_white_string( const std::string& );
+   void         split          ( const std::string&, std::list<std::string>&, const char = ' ' );
    std::string& to_lower       ( std::string& ) const;
 
    //======= ltrim =======
