@@ -34,6 +34,7 @@
 #include "QBtShared.h"
 #include "QBtEventsController.h"
 #include "QBtOperator.h"
+#include "QBtNumeration.h"
 #include <QFile>
 #include <QFileInfo>
 #include <QFont>
@@ -61,7 +62,7 @@ const char* const QBtBrowser::SAVE_ERROR  = QT_TR_NOOP( "Error on save the file:
 //*******************************************************************
 // QBtBrowser                                            CONSTRUCTOR
 //*******************************************************************
-QBtBrowser::QBtBrowser( const int      in_is_left,
+QBtBrowser::QBtBrowser( const int         in_is_left,
                         QScrollBar* const in_scroll,
                         QWidget* const    in_parent )
 : QTextBrowser ( in_parent )
@@ -69,13 +70,16 @@ QBtBrowser::QBtBrowser( const int      in_is_left,
 , scroll_      ( in_scroll )
 , syntax_      ( new QBtSyntax( document() ) )
 , operator_    ( 0 )
+, numeration_  ( new QBtNumeration( this ) )
 , fpath_       ( QString() )
 , drop_fpath_  ( QString() )
 , is_writable_ ( true )
-, saved_sbpos_ ( int() )
-, is_changed_  ( int() )
+, saved_nr1_   ( -1 )
+, saved_nr2_   ( -1 )
+, is_changed_  ( false )
 {
    setAcceptDrops( true );
+   setAcceptRichText( false );
    
    setFrameShadow( Plain );
    setFrameShape( NoFrame );
@@ -86,30 +90,111 @@ QBtBrowser::QBtBrowser( const int      in_is_left,
    update_looks();
    
    init_scrollbar();
+
+   set_numeration( QBtConfig::instance()->numeration() );
+
+   QBtEventsController* const ec = QBtEventsController::instance();
+   ec->append( this, QBtEvent::NUMERATION );
+   ec->append( this, QBtEvent::DOCUMENT_CHANGED );
 }
 // end of QBtBrowser
 
 //*******************************************************************
-// ~QBtBrowser                                           DESTRUCTOR
+// ~QBtBrowser                                            DESTRUCTOR
 //*******************************************************************
 QBtBrowser::~QBtBrowser()
-{}
+{
+   QBtEventsController::instance()->remove( this );
+}
 // end of ~QBtBrowser
 
+//*******************************************************************
+// updates_enable                                             PUBLIC
+//*******************************************************************
+void QBtBrowser::updates_enable( const bool in_stat )
+{
+   setUpdatesEnabled( in_stat );
+   scroll_->setUpdatesEnabled( in_stat );
+}
+// end of updates_enable
+
+//*******************************************************************
+// resizeEvent                                     PRIVATE inherited
+//*******************************************************************
+void QBtBrowser::resizeEvent( QResizeEvent* const in_event )
+{
+   QTextBrowser::resizeEvent( in_event );
+   update_numeration();   
+}
+// end of resizeEvent
+
+//*******************************************************************
+// customEvent                                     PRIVATE inherited
+//*******************************************************************
+void QBtBrowser::customEvent( QEvent* const in_event )
+{
+   QBtEvent* const event = dynamic_cast< QBtEvent* >( in_event );
+   const int type = static_cast< int >( event->type() );
+
+   switch( type ) {
+      case QBtEvent::NUMERATION:
+         {
+            const bool state = event->data( 0 ).toBool();
+            QBtConfig::instance()->numeration( state );
+            QBtConfig::instance()->save_data();
+            set_numeration( state );
+         }
+         break;
+   }
+}
+// end of customEvent
+
+//*******************************************************************
+// set_numeration                                            PRIVATE
+//*******************************************************************
+void QBtBrowser::set_numeration( const bool in_visible )
+{
+   numeration_->setEnabled( in_visible );
+   numeration_->setVisible( in_visible );
+   update_numeration();
+   update();
+}
+// end of set_numeration
+
 //*******************************************************************
 // update_looks                                              PRIVATE
 //*******************************************************************
 void QBtBrowser::update_looks()
 {
-   QPalette p = viewport()->palette();
-   p.setColor( viewport()->backgroundRole(), QBtConfig::instance()->bkg_color() );
-   viewport()->setPalette( p );
+   const QBtConfig* const cfg = QBtConfig::instance();
+   
+   {
+      QPalette p = viewport()->palette();
+      p.setColor( viewport()->backgroundRole(), cfg->text_background() );
+      viewport()->setPalette( p );
+   }
+   {
+      QPalette p = palette();
+      p.setColor( backgroundRole(), cfg->text_background() );
+      setPalette( p );
+   }  
+
    syntax_->renew();
-   document()->adjustSize();
    renew();
 }
 // end of update_looks
 
+//*******************************************************************
+// update_numeration                                         PRIVATE
+//*******************************************************************
+void QBtBrowser::update_numeration()
+{
+   numeration_->update_looks();
+   numeration_->update_geometry();
+   setViewportMargins( numeration_->width(), 0, 0, 0 );
+}
+// end of update_numeration
+
 //*******************************************************************
 // read_file                                                  PUBLIC
 //*******************************************************************
@@ -130,44 +215,46 @@ bool QBtBrowser::read_file( const QString& in_fpath, const bool in_is_drag )
    if( !fi.isWritable() ) {
       QMessageBox::information( this, tr( SAVING ), tr( READ_ONLY ).arg( fpath ) );
    }
+
    QFile file( fpath );
    if( file.open( QIODevice::ReadOnly ) ) {
+      QBtShared::set_cursor( Qt::WaitCursor );
       clear();
-      QApplication::setOverrideCursor( Qt::WaitCursor );
       const QByteArray data = file.readAll();
-      setPlainText( QString::fromLocal8Bit( data ) );
-      QApplication::restoreOverrideCursor();
-      
-      file.close();
+      if( QBtConfig::instance()->utf8() ) {
+         setPlainText( QString::fromUtf8( data ) );
+      }
+      else {
+         setPlainText( QString::fromLocal8Bit( data ) );
+      }
+
+       file.close();
       fpath_ = fpath;
       is_writable_ = fi.isWritable();
+      changed( false );
+      
+      renew();
+      if( in_is_drag ) emit update_request();
       document()->adjustSize();
-   }
-   else {
-      QMessageBox::critical( this, tr( READING ),
-         tr( READ_ERROR ).arg( fpath ).arg( file.errorString() ) );
-      return false;
+      QBtShared::restore_cursor();
+      return true;
    }
 
-   if( in_is_drag ) {
-      emit update_request();
-   }
-   return true;
+   QMessageBox::critical( this, tr( READING ), tr( READ_ERROR ).arg( fpath ).arg( file.errorString() ) );
+   return false;
 }
 // end of read_file
 
 //*******************************************************************
 // save_file                                                  PUBLIC
 //*******************************************************************
-void QBtBrowser::save_file()
+bool QBtBrowser::save_file()
 {
-   if( fpath_.isEmpty() ) return;
-   if( !is_changed_ ) return;
+   if( fpath_.isEmpty() ) return false;
+   if( !is_changed_ ) return true;
    
    const QFileInfo fi( fpath_ );
-   if( !fi.isWritable() ) {
-      return;
-   }
+   if( !fi.isWritable() ) return false;
 
    // Tworzymy kopie zapasowa starego pliku.
    const QString tmp_fpath = fpath_ + ".bak";
@@ -181,12 +268,12 @@ void QBtBrowser::save_file()
       file.write( toPlainText().toLocal8Bit() );
       QApplication::restoreOverrideCursor();
       file.close();
-      is_changed_ = 0;
-   }
-   else {
-      QMessageBox::critical( this, tr( SAVING ),
-         tr( SAVE_ERROR ).arg( fpath_ ).arg( file.errorString() ) );
+      changed( false );
+      return true;
    }
+   
+   QMessageBox::critical( this, tr( SAVING ), tr( SAVE_ERROR ).arg( fpath_ ).arg( file.errorString() ) );
+   return false;
 }
 // end of save_file
 
@@ -212,11 +299,26 @@ void QBtBrowser::renew( const bool in_repaint )
       block = block.next();
       ++n;
    }
-   
-   if( in_repaint ) viewport()->repaint();
+
+   update_numeration();
+   if( in_repaint ) {
+      viewport()->repaint();
+      numeration_->repaint();
+   }
 }
 // end of renew
 
+//*******************************************************************
+// changed                                                   PRIVATE
+//*******************************************************************
+void QBtBrowser::changed( const bool in_is_changed )
+{
+   is_changed_ = in_is_changed;
+   QBtEventsController* const ec = QBtEventsController::instance();
+   ec->send_event( QBtEvent::DOCUMENT_CHANGED, is_left_, is_changed_ );
+}
+// end of changed
+
 //*******************************************************************
 // fragment_position                                         PRIVATE
 //*******************************************************************
@@ -248,9 +350,9 @@ void QBtBrowser::fragment_position( const int  in_nr1,
 //*******************************************************************
 // remove_range                                               PUBLIC
 //*******************************************************************
-void QBtBrowser::remove_range( const int in_nr1,
-                               const int in_nr2,
-                               const bool   in_update )
+void QBtBrowser::remove_range( const int  in_nr1,
+                               const int  in_nr2,
+                               const bool in_update )
 {
    if( in_update ) save_pos();
    
@@ -265,22 +367,21 @@ void QBtBrowser::remove_range( const int in_nr1,
       cursor.setPosition( pos2, QTextCursor::KeepAnchor );
       cursor.removeSelectedText();
       cursor.endEditBlock();
+      changed( true );
    }
    if( in_update ) {
       document()->adjustSize();
       restore_pos();
    }
-   is_changed_ = 1;
 }
 // end of remove_range
 
 //*******************************************************************
 // get_fragment                                               PUBLIC
 //*******************************************************************
-QTextDocumentFragment QBtBrowser::get_fragment( const int in_nr1,
-                                                const int in_nr2 )
+QString QBtBrowser::get_fragment( const int in_nr1, const int in_nr2 )
 {
-   QTextDocumentFragment retval = QTextDocumentFragment();
+   QString retval = QString();
    
    int pos1 = -1;
    int pos2 = -1;
@@ -290,7 +391,7 @@ QTextDocumentFragment QBtBrowser::get_fragment( const int in_nr1,
       QTextCursor cursor = textCursor();
       cursor.setPosition( pos1 );
       cursor.setPosition( pos2, QTextCursor::KeepAnchor );
-      retval = cursor.selection();
+      retval = cursor.selectedText();
    }
    return retval;
 }
@@ -299,9 +400,9 @@ QTextDocumentFragment QBtBrowser::get_fragment( const int in_nr1,
 //*******************************************************************
 // set_fragment                                               PUBLIC
 //*******************************************************************
-void QBtBrowser::set_fragment( const int in_nr,
-                               const QTextDocumentFragment& in_fragment,
-                               const bool in_update )
+void QBtBrowser::set_fragment( const int      in_nr,
+                               const QString& in_text,
+                               const bool     in_update )
 {
    if( in_update ) save_pos();
    
@@ -312,7 +413,7 @@ void QBtBrowser::set_fragment( const int in_nr,
       // Jest to sytuacja specjalna.
       // Normalnie nie ma linii 0. Linia 0 to pseudo-linia.
       // Przyslanie numeru zero oznacza, ze tekst nalezy wcisnac
-      // przed 1-sza linia tekstu. Na poczatek tekstu.
+      // przed 1-sza linia. Na poczatek tekstu.
       pos1 = pos2 = 0;
    }
    else {
@@ -325,29 +426,30 @@ void QBtBrowser::set_fragment( const int in_nr,
       QTextCursor cursor = textCursor();
       cursor.beginEditBlock();
       cursor.setPosition( pos2 );
-      cursor.insertFragment( in_fragment );
+      cursor.insertText( in_text );
       cursor.endEditBlock();
+      changed( true );
    }
    if( in_update ) {
-      document()->adjustSize();
+       document()->adjustSize();
       restore_pos();
    }
-   is_changed_ = 1;
 }
 // end of set_fragment
 
 //*******************************************************************
 // replace_fragment                                           PUBLIC
 //*******************************************************************
-void QBtBrowser::replace_fragment( const int in_nr1,
-                                   const int in_nr2,
-                                   const QTextDocumentFragment& in_fragment,
-                                   const bool   in_update )
+void QBtBrowser::replace_fragment( const int      in_nr1,
+                                   const int      in_nr2,
+                                   const QString& in_text,
+                                   const bool     in_update )
 {
    if( in_update ) save_pos();
    
    int pos1 = -1;
    int pos2 = -1;
+   
    fragment_position( in_nr1, in_nr2, pos1, pos2 );
 
    if( ( pos1 != -1 ) && ( pos2 != -1 ) ) {
@@ -355,14 +457,14 @@ void QBtBrowser::replace_fragment( const int in_nr1,
       cursor.beginEditBlock();
       cursor.setPosition( pos1 );
       cursor.setPosition( pos2, QTextCursor::KeepAnchor );
-      cursor.insertFragment( in_fragment );
+      cursor.insertText( in_text );
       cursor.endEditBlock();
+      changed( true );
    }
    if( in_update ) {
       document()->adjustSize();
       restore_pos();
    }
-   is_changed_ = 1;
 }
 // end of replace_fragment
 
@@ -374,12 +476,14 @@ QString QBtBrowser::get_y( const int in_nr1,
                            int&      out_yt,
                            int&      out_yb ) const
 {
+ 
    QString retval = QString();
    
    QTextBlock block = document()->begin();
    while( block.isValid() ) {
       const QBtLineData* info = dynamic_cast< QBtLineData* >( block.userData() );
       const int nr = info->get_number();
+
       if( nr < in_nr1 ) {
          block = block.next();
          continue;
@@ -412,48 +516,49 @@ QString QBtBrowser::get_y( const int in_nr1,
 //*******************************************************************
 void QBtBrowser::paintEvent( QPaintEvent* const in_event )
 {
-   setTextColor( QBtConfig::instance()->font_color() );
-   QPainter p( viewport() );
-
    const QBtConfig* const cfg = QBtConfig::instance();
    
+   setTextColor( cfg->text_color() );
+   QPainter p( viewport() );
+   
    const int view_yt    = verticalScrollBar()->value();
+   const int check_yt   = view_yt - 1;
    const int view_yb    = view_yt + viewport()->height() - 1;
+   const int check_yb   = view_yb + 1;
    const int width      = viewport()->width();
-   bool         is_painted = false;
 
    QTextBlock block = document()->begin();
    while( block.isValid() ) {
       const int block_yt = qRound( block.layout()->position().y() );
       const int block_h  = block.layout()->boundingRect().toRect().height();
       const int block_yb = block_yt + block_h - 1;
-
-      if( !is_inside( view_yt - 1, view_yb + 1, block_yt, block_yb ) ) {
-         if( is_painted ) break;
+      
+      if( block_yb < check_yt ) {
          block = block.next();
          continue;
       }
+      if( block_yt > check_yb ) break;
+      
 
-      is_painted = true;
       QBtLineData* const info = dynamic_cast<QBtLineData*>( block.userData() );
       if( info ) {
          const int block_yt = qRound( block.layout()->position().y() );
          const int block_h  = block.layout()->boundingRect().toRect().height();
-         QBrush brush          = cfg->bkg_brush();
+         QBrush brush       = cfg->text_background_brush();
 
          if( info->get_status() ) {
             bool only_line = false;
             switch( info->get_status() ) {
                case QBtShared::CHANGE:
-                  brush = cfg->chg_brush();
+                  brush = cfg->chg_background_brush();
                   break;   
                case QBtShared::DELETE:
-                  brush = cfg->del_brush();
-                  if( !is_left_ ) only_line = true;
+                  brush = cfg->del_background_brush();
+                  only_line = !is_left_;
                   break;
                case QBtShared::APPEND:
-                  brush = cfg->add_brush();
-                  if( is_left_ ) only_line = true;
+                  brush = cfg->add_background_brush();
+                  only_line = is_left_;
                   break;
             }
             if( only_line ) {
@@ -482,7 +587,9 @@ void QBtBrowser::paintEvent( QPaintEvent* const in_event )
                QBtDiffInfo di = is_left_ ? dp->info_for_nr_in_first( 0 )
                                          : dp->info_for_nr_in_second( 0 );
                if( di.is_valid() ) {
-                  const QBrush brush = is_left_ ? cfg->add_brush() : cfg->del_brush();
+                  const QBrush brush = is_left_
+                                     ? cfg->add_background_brush()
+                                     : cfg->del_background_brush();
                   p.fillRect( QRect( 0, 0, width, 2 ), brush );
                }
             }
@@ -491,24 +598,9 @@ void QBtBrowser::paintEvent( QPaintEvent* const in_event )
       block = block.next();
    }
    p.end();
-   
    QTextBrowser::paintEvent( in_event );
 }
-// end of renew
-
-//*******************************************************************
-// is_inside                                                 PRIVATE
-//*******************************************************************
-bool QBtBrowser::is_inside( const int in_view_yt,
-                            const int in_view_yb,
-                            const int in_block_yt,
-                            const int in_block_yb ) const
-{
-   if( ( in_block_yt >= in_view_yt ) && ( in_block_yt <= in_view_yb ) ) return true;
-   if( ( in_block_yb >= in_view_yt ) && ( in_block_yb <= in_view_yb ) ) return true;
-   return false;
-}
-// end of in_inside
+// end of paintEvent
 
 //*******************************************************************
 // goto_line                                                  PUBLIC
@@ -537,20 +629,18 @@ void QBtBrowser::goto_line( const int in_nr )
 //*******************************************************************
 void QBtBrowser::set_cursor_position( const int in_position )
 {
-   if( in_position >= 0 ) {
-      QTextCursor cursor = textCursor();
-      cursor.setPosition( in_position );
-      setTextCursor( cursor );
-      ensureCursorVisible();
-   }
+   QTextCursor cursor = textCursor();
+   cursor.setPosition( in_position );
+   setTextCursor( cursor );
+   ensureCursorVisible();
 }
 // end of set_cursor_position
 
 //*******************************************************************
 // first_selected                                             PUBLIC
 //-------------------------------------------------------------------
-// Zwraca pierwsza WYBRANA linie. Wybrana tzn. taka ktora jest
-// wskazana przez diff'a.
+// Zwraca PIERWSZA WYBRANA linie WIDOCZNA W CALOSCI a ekranie.
+// Wybrana tzn. taka ktora jest wskazana przez diff'a.
 //*******************************************************************
 int QBtBrowser::first_selected() const
 {
@@ -558,8 +648,10 @@ int QBtBrowser::first_selected() const
    const int view_yb = view_yt + viewport()->height() - 1;
    
    QTextBlock block = document()->begin();
-   while( block.isValid() && block.userData() ) {
-      const QBtLineData* const info = dynamic_cast< QBtLineData* >( block.userData() );
+   while( block.isValid() ) {
+      const QBtLineData* const info = dynamic_cast<QBtLineData*>( block.userData() );
+      if( !info ) return -1;
+      
       const int block_yt = qRound( block.layout()->position().y() );
       const int block_h  = block.layout()->boundingRect().toRect().height();
       const int block_yb = block_yt + block_h - 1;
@@ -577,6 +669,9 @@ int QBtBrowser::first_selected() const
 
 //*******************************************************************
 // last_selected                                              PUBLIC
+//-------------------------------------------------------------------
+// Zwraca OSTATNIA WYBRANA linie WIDOCZNA W CALOSCI na ekranie.
+// Wybrana tzn. taka ktora jest wskazana przez diff'a.
 //*******************************************************************
 int QBtBrowser::last_selected() const
 {
@@ -585,7 +680,7 @@ int QBtBrowser::last_selected() const
    
    QTextBlock block = document()->end().previous();
    while( block.isValid() ) {
-      const QBtLineData* const info = dynamic_cast< QBtLineData* >( block.userData() );
+      const QBtLineData* const info = dynamic_cast<QBtLineData*>( block.userData() );
       if( !info ) return -1;
       
       const int block_yt = qRound( block.layout()->position().y() );
@@ -605,6 +700,9 @@ int QBtBrowser::last_selected() const
 
 //*******************************************************************
 // first_visible                                              PUBLIC
+//-------------------------------------------------------------------
+// Zwaraca PIERWSZA WIDOCZNA W CALOSCI linie.
+// Czy jest zaznaczona, czy nie, nie ma tu nic do rzeczy.
 //*******************************************************************
 int QBtBrowser::first_visible() const
 {
@@ -612,7 +710,7 @@ int QBtBrowser::first_visible() const
    
    QTextBlock block = document()->begin();
    while( block.isValid() ) {
-      const QBtLineData* const info = dynamic_cast< QBtLineData* >( block.userData() );
+      const QBtLineData* const info = dynamic_cast<QBtLineData*>( block.userData() );
       if( !info ) return -1;
 
       const int block_yt = qRound( block.layout()->position().y() );
@@ -627,6 +725,9 @@ int QBtBrowser::first_visible() const
 
 //*******************************************************************
 // last_visible                                               PUBLIC
+//-------------------------------------------------------------------
+// Zwraca OSTATNIA WIDOCZNA W CALOSCI linie.
+// Czy jest zaznaczona, czy nie, nie ma tu nic do rzeczy.
 //*******************************************************************
 int QBtBrowser::last_visible() const
 {
@@ -650,6 +751,15 @@ int QBtBrowser::last_visible() const
 }
 // end of last_visible
 
+//*******************************************************************
+// numeration_update                                          PUBLIC
+//*******************************************************************
+void QBtBrowser::numeration_update()
+{
+   numeration_->update();
+}
+// end of numeration_update
+
 
 //###################################################################
 //#                                                                 #
@@ -712,7 +822,7 @@ void QBtBrowser::set_range( int in_min, int in_max ) const
 {
    scroll_->setRange( in_min, in_max );
    scroll_->setPageStep( verticalScrollBar()->pageStep() );
-   scroll_->setVisible( in_max != 0 );
+   scroll_->setVisible( in_max );
 }
 // end of set_range
 
@@ -730,12 +840,10 @@ void QBtBrowser::set_value( const int in_value ) const
 //*******************************************************************
 void QBtBrowser::dragEnterEvent( QDragEnterEvent* const in_event )
 {
-   if( in_event ) {
-      const QMimeData* const md = in_event->mimeData();
-      if( md ) {
-         if( md->hasText() ) in_event->accept();
-      }
-   }
+   if( !in_event ) return;
+   
+   const QMimeData* const md = in_event->mimeData();
+   if( md ) if( md->hasUrls() ) in_event->accept();
 }
 // end of dragEnterEvent
 
@@ -744,12 +852,10 @@ void QBtBrowser::dragEnterEvent( QDragEnterEvent* const in_event )
 //*******************************************************************
 void QBtBrowser::dragMoveEvent( QDragMoveEvent* const in_event )
 {
-   if( in_event ) {
-      const QMimeData* const md = in_event->mimeData();
-      if( md ) {
-         if( md->hasText() ) in_event->accept();
-      }
-   }
+   if( !in_event ) return;
+   
+   const QMimeData* const md = in_event->mimeData();
+   if( md ) if( md->hasUrls() ) in_event->accept();
 }
 // end of dragMoveEvent
 
@@ -758,11 +864,57 @@ void QBtBrowser::dragMoveEvent( QDragMoveEvent* const in_event )
 //*******************************************************************
 void QBtBrowser::dropEvent( QDropEvent* const in_event )
 {
-   const QMimeData* const mdata = in_event->mimeData();
-   if( !mdata ) return;
-   if( !mdata->hasText() ) return;
+   if( !in_event ) return;
+   
+   const QMimeData* const md = in_event->mimeData();
+   if( !md ) return;
+   if( !md->hasUrls() ) return;
 
-   const QFileInfo fi( QUrl( mdata->text().trimmed() ).toLocalFile() );
+   const QFileInfo fi( md->urls().first().toLocalFile() );
    if( fi.isFile() ) read_file( drop_fpath_ = fi.absoluteFilePath(), true );
 }
 // end of dropEvent
+
+//*******************************************************************
+// save_pos                                                  PRIVATE
+//*******************************************************************
+void QBtBrowser::save_pos()
+{
+   saved_nr1_ = first_visible();
+   saved_nr2_ = last_visible();
+   //qDebug() << "save_pos: " << saved_nr1_ << ", " << saved_nr2_;
+}   
+// end of save_pos
+
+//*******************************************************************
+// restore_pos                                               PRIVATE
+//*******************************************************************
+void QBtBrowser::restore_pos()
+{
+   int first_nr = -1;
+   int last_nr  = -1;
+
+   {
+      QTextBlock block = document()->begin();
+      if( block.isValid() ) {
+         const QBtLineData* const info = dynamic_cast< QBtLineData* >( block.userData() );
+         if( info ) first_nr = info->get_number();
+      }
+   }
+   {   
+      QTextBlock block = document()->end().previous();
+      if( block.isValid() ) {
+         const QBtLineData* const info = dynamic_cast< QBtLineData* >( block.userData() );
+         if( info ) last_nr = info->get_number();
+      }
+   }
+
+   first_nr = ( first_nr != -1 ) ? qMin( last_nr, saved_nr1_ ) : saved_nr1_;
+   last_nr  = ( last_nr  != -1 ) ? qMin( last_nr , saved_nr2_ ) : saved_nr2_;
+
+   //qDebug() << "restore_pos: " << first_nr << ", " << last_nr;
+
+   goto_line( first_nr );
+   goto_line( last_nr  );
+}
+// end of restore_pos
