@@ -177,8 +177,8 @@ void QBtDiffProcess::parse()
 //*******************************************************************
 int QBtDiffProcess::in_first( const int in_nr )
 {
-   QVector< QBtDiffInfo >::iterator it = ranges_.begin();
-   const QVector< QBtDiffInfo >::iterator end = ranges_.end();
+   QVector<QBtDiffInfo>::iterator it = ranges_.begin();
+   const QVector<QBtDiffInfo>::iterator end = ranges_.end();
    while( it != end ) {
       if( it->first_range().is_nr_in_range( in_nr ) ) {
          return it->first_range().oper();
@@ -194,8 +194,8 @@ int QBtDiffProcess::in_first( const int in_nr )
 //*******************************************************************
 int QBtDiffProcess::in_second( const int in_nr )
 {
-   QVector< QBtDiffInfo >::iterator it = ranges_.begin();
-   const QVector< QBtDiffInfo >::iterator end = ranges_.end();
+   QVector<QBtDiffInfo>::iterator it = ranges_.begin();
+   const QVector<QBtDiffInfo>::iterator end = ranges_.end();
    while( it != end ) {
       if( it->second_range().is_nr_in_range( in_nr ) ) {
          return it->second_range().oper();
@@ -256,8 +256,6 @@ int QBtDiffProcess::second_to_first( const int in_nr )
 void QBtDiffProcess::insert_c( const QString& in_range )
 {
    const int idx = in_range.indexOf( 'c' );
-   if( -1 == idx ) return;
-
    const QString first = in_range.left( idx );
    const QString second = in_range.mid( idx + 1 );
    ranges_ << QBtDiffInfo( first, QBtShared::CHANGE, second );
@@ -271,8 +269,6 @@ void QBtDiffProcess::insert_c( const QString& in_range )
 void QBtDiffProcess::insert_a( const QString& in_range )
 {
    const int idx = in_range.indexOf( 'a' );
-   if( -1 == idx ) return;
-
    const QString first = in_range.left( idx );
    const QString second = in_range.mid( idx + 1 );
    ranges_ << QBtDiffInfo( first, QBtShared::APPEND, second );
@@ -286,8 +282,6 @@ void QBtDiffProcess::insert_a( const QString& in_range )
 void QBtDiffProcess::insert_d( const QString& in_range )
 {
    const int idx = in_range.indexOf( 'd' );
-   if( -1 == idx ) return;
-
    const QString first = in_range.left( idx );
    const QString second = in_range.mid( idx + 1 );
    ranges_ << QBtDiffInfo( first, QBtShared::DELETE, second );
@@ -308,16 +302,15 @@ void QBtDiffProcess::update(  QBtBrowser* const in_left_browser,
    const QVector< QBtDiffInfo >::iterator end = ranges_.end();
    while( it != end ) {
       QBtRange& range_on_left  = it->first_range();
-      QBtRange& range_on_right = it->second_range();
       const int nr1_1 = range_on_left.nr1();
       const int nr1_2 = range_on_left.nr2();
-      const int nr2_1 = range_on_right.nr1();
-      const int nr2_2 = range_on_right.nr2();
-
       const QString txt1 = in_left_browser->get_y( nr1_1, nr1_2, yt, yb );
       range_on_left.yt( yt );
       range_on_left.yb( yb );
-      
+
+      QBtRange& range_on_right = it->second_range();
+      const int nr2_1 = range_on_right.nr1();
+      const int nr2_2 = range_on_right.nr2();
       const QString txt2 = in_right_browser->get_y( nr2_1, nr2_2, yt, yb );
       range_on_right.yt( yt );
       range_on_right.yb( yb );
