@@ -44,6 +44,7 @@
 #include <QMessageBox>
 #include <QStatusBar>
 #include <QCloseEvent>
+#include <QtDebug>
 
 /*------- local constants:
 -------------------------------------------------------------------*/
@@ -51,8 +52,10 @@ const char* const QBtMainWindow::SYSTEM         = QT_TR_NOOP( "&System" );
 const char* const QBtMainWindow::OPERATIONS     = QT_TR_NOOP( "&Operations" );
 const char* const QBtMainWindow::HELP           = QT_TR_NOOP( "&Help" );
 //------- SYSTEM -------
-const char* const QBtMainWindow::REFRESH        = QT_TR_NOOP( "&Refresh" );
-const char* const QBtMainWindow::REFRESH_SKEY   = QT_TR_NOOP( "Ctrl+R" );
+const char* const QBtMainWindow::SYNC           = QT_TR_NOOP( "&Synchronized scroll" );
+const char* const QBtMainWindow::SYNC_KEY       = QT_TR_NOOP( "Ctrl+S" ); 
+const char* const QBtMainWindow::NUMERATION     = QT_TR_NOOP( "&Numeration" );
+const char* const QBtMainWindow::NUMERATION_KEY = QT_TR_NOOP( "Ctrl+N" ); 
 const char* const QBtMainWindow::SAVE           = QT_TR_NOOP( "&Save" );
 const char* const QBtMainWindow::SAVE_SKEY      = QT_TR_NOOP( "Ctrl+S" );
 const char* const QBtMainWindow::CONFIGURE      = QT_TR_NOOP( "&Configure" );
@@ -65,8 +68,6 @@ const char* const QBtMainWindow::MERGE_TO_RIGHT = QT_TR_NOOP( "Merge all to &rig
 const char* const QBtMainWindow::DEL_ON_LEFT    = QT_TR_NOOP( "Re&move all from left" );
 const char* const QBtMainWindow::DEL_ON_RIGHT   = QT_TR_NOOP( "Remo&ve all from right" );
 //------- HELP -------
-const char* const QBtMainWindow::ABOUT_ICON     = ":/img/beediff.png";
-const char* const QBtMainWindow::ABOUT_QT_ICON  = ":/img/qt.png";
 const char* const QBtMainWindow::ABOUT          = QT_TR_NOOP( "&About..." );
 const char* const QBtMainWindow::ABOUT_QT       = QT_TR_NOOP( "About &Qt..." );
 //------- OTHER -------
@@ -74,6 +75,18 @@ const char* const QBtMainWindow::STAT_TOTAL     = QT_TR_NOOP( "Total: %1" );
 const char* const QBtMainWindow::STAT_CHG       = QT_TR_NOOP( "Changed: %1" );
 const char* const QBtMainWindow::STAT_ADD       = QT_TR_NOOP( "Added: %1" );
 const char* const QBtMainWindow::STAT_DEL       = QT_TR_NOOP( "Deleted: %1");
+//------- ICONS -------
+const QString QBtMainWindow::SYNC_ICON       = ":/img/sync.png";
+const QString QBtMainWindow::NUMERATION_ICON = ":/img/numbers.png";
+const QString QBtMainWindow::DISK_ICON       = ":/img/floppy.png";
+const QString QBtMainWindow::CONFIG_ICON     = ":/img/configure.png";
+const QString QBtMainWindow::EXIT_ICON       = ":/img/exit.png";
+const QString QBtMainWindow::MERGE2LFT_ICON  = ":/img/merge_to_left.png";
+const QString QBtMainWindow::MERGE2RGT_ICON  = ":/img/merge_to_right.png";
+const QString QBtMainWindow::DELLFT_ICON     = ":/img/del_on_left.png";
+const QString QBtMainWindow::DELRGT_ICON     = ":/img/del_on_right.png";
+const QString QBtMainWindow::ABOUT_ICON      = ":/img/beediff.png";
+const QString QBtMainWindow::ABOUT_QT_ICON   = ":/img/qt.png";
 
 
 //*******************************************************************
@@ -88,18 +101,19 @@ QBtMainWindow::QBtMainWindow() : QMainWindow()
 , system_bar_     ( new QToolBar )
 , operations_bar_ ( new QToolBar )
 //------- system -------
-, refresh_        ( new QAction( QIcon( ":/img/refresh.png" ), "", this ) )
-, save_           ( new QAction( QIcon( ":/img/floppy.png" ), "", this ) )
-, configure_      ( new QAction( QIcon( ":/img/configure.png" ), "", this ) )
-, exit_           ( new QAction( QIcon( ":/img/exit.png" ), "", this ) )
+, sync_           ( new QAction( QIcon( SYNC_ICON       ), QString(), this ) )
+, numeration_     ( new QAction( QIcon( NUMERATION_ICON ), QString(), this ) )
+, save_           ( new QAction( QIcon( DISK_ICON       ), QString(), this ) )
+, configure_      ( new QAction( QIcon( CONFIG_ICON     ), QString(), this ) )
+, exit_           ( new QAction( QIcon( EXIT_ICON       ), QString(), this ) )
 //------- operations -------
-, merge_to_left_  ( new QAction( QIcon( ":/img/merge_to_left.png" ), "", this ) )
-, merge_to_right_ ( new QAction( QIcon( ":/img/merge_to_right.png" ), "", this ) )
-, del_on_left_    ( new QAction( QIcon( ":/img/del_on_left.png" ), "", this ) )
-, del_on_right_   ( new QAction( QIcon( ":/img/del_on_right.png" ), "", this ) )
+, merge_to_left_  ( new QAction( QIcon( MERGE2LFT_ICON  ), QString(), this ) )
+, merge_to_right_ ( new QAction( QIcon( MERGE2RGT_ICON  ), QString(), this ) )
+, del_on_left_    ( new QAction( QIcon( DELLFT_ICON     ), QString(), this ) )
+, del_on_right_   ( new QAction( QIcon( DELRGT_ICON     ), QString(), this ) )
 //------- help -------
-, about_          ( new QAction( QIcon( ABOUT_ICON ), "", this ) )
-, about_qt_       ( new QAction( QIcon( ABOUT_QT_ICON ), "", this ) )
+, about_          ( new QAction( QIcon( ABOUT_ICON    ), QString(), this ) )
+, about_qt_       ( new QAction( QIcon( ABOUT_QT_ICON ), QString(), this ) )
 , workspace_      ( new QBtWorkspace( this ) )
 //------- other -------
 , stat_total_     ( new QLabel )
@@ -108,6 +122,11 @@ QBtMainWindow::QBtMainWindow() : QMainWindow()
 , stat_del_       ( new QLabel )
 {
    save_->setEnabled( false );
+   sync_->setCheckable( true );
+   sync_->setChecked( QBtConfig::instance()->sync() );
+   numeration_->setCheckable( true );
+   numeration_->setChecked( QBtConfig::instance()->numeration() );
+   
    del_on_left_->setEnabled( false );
    del_on_right_->setEnabled( false );
    merge_to_left_->setEnabled( false );
@@ -253,7 +272,9 @@ void QBtMainWindow::create_menu()
    menuBar()->addMenu( help_   );
    
    //------- system -------
-   system_->addAction( refresh_ );
+   system_->addAction( sync_ );
+   system_->addAction( numeration_ );
+   system_->addSeparator();
    system_->addAction( save_ );
    system_->addAction( configure_ );
    system_->addSeparator();
@@ -281,7 +302,8 @@ void QBtMainWindow::create_tbar()
    system_bar_->addAction( exit_ );
    system_bar_->addSeparator();
    system_bar_->addAction( save_ );
-   system_bar_->addAction( refresh_ );
+   system_bar_->addAction( sync_ );
+   system_bar_->addAction( numeration_ );
 
    operations_bar_->addAction( del_on_left_ );
    operations_bar_->addAction( del_on_right_ );
@@ -294,55 +316,41 @@ void QBtMainWindow::create_tbar()
 //*******************************************************************
 // create_sbar                                               PRIVATE
 //*******************************************************************
-void QBtMainWindow::create_sbar()
+void QBtMainWindow::create_sbar() const
 {
-   static const QString STRNR = "9999x";
-   QStatusBar* const sbar = statusBar();
-   const QFontMetrics fm( statusBar()->font() );
-   
-   {
-      const int dx = fm.width( tr( STAT_TOTAL ).arg( STRNR ) );
-      stat_total_->setIndent( 3 );
-      stat_total_->setMinimumWidth( dx );
-      stat_total_->setMaximumWidth( dx );
-      sbar->addPermanentWidget( stat_total_ );
-   }
-   {
-      const int dx = fm.width( tr( STAT_CHG ).arg( STRNR ) );
-      stat_chg_->setIndent( 3 );
-      stat_chg_->setMinimumWidth( dx );
-      stat_chg_->setMaximumWidth( dx );
-      sbar->addPermanentWidget( stat_chg_ );
-   }
-   {
-      const int dx = fm.width( tr( STAT_ADD ).arg( STRNR ) );
-      stat_add_->setIndent( 3 );
-      stat_add_->setMinimumWidth( dx );
-      stat_add_->setMaximumWidth( dx );
-      sbar->addPermanentWidget( stat_add_ );
-   }
-   {
-      const int dx = fm.width( tr( STAT_DEL ).arg( STRNR ) );
-      stat_del_->setIndent( 3 );
-      stat_del_->setMinimumWidth( dx );
-      stat_del_->setMaximumWidth( dx );
-      sbar->addPermanentWidget( stat_del_ );
-   }
-   
-   sbar->setSizeGripEnabled( true );
+   set_sbar_widget( tr( STAT_TOTAL ), stat_total_ );
+   set_sbar_widget( tr( STAT_CHG   ), stat_chg_   );
+   set_sbar_widget( tr( STAT_ADD   ), stat_add_   );
+   set_sbar_widget( tr( STAT_DEL   ), stat_del_   );
 }
 // end of create_sbar
 
+//*******************************************************************
+// set_sbar_widget                                           PRIVATE
+//*******************************************************************
+void QBtMainWindow::set_sbar_widget( const QString& in_text, QLabel* const in_widget ) const
+{
+   QStatusBar* const sbar = statusBar();
+   
+   const int dx = QFontMetrics( sbar->font() ).width( in_text.arg( "9999" ) );
+   in_widget->setIndent( 3 );
+   in_widget->setMinimumWidth( dx );
+   in_widget->setMaximumWidth( dx );
+   sbar->addPermanentWidget( in_widget );
+}
+// end of set_sbar_widget
+
 //*******************************************************************
 // update_actions                                            PRIVATE
 //*******************************************************************
 void QBtMainWindow::update_actions()
 {
    //------- system -------
-   connect( refresh_  , SIGNAL( triggered() ), this, SLOT( refresh  () ) );
-   connect( save_     , SIGNAL( triggered() ), this, SLOT( save     () ) );
-   connect( configure_, SIGNAL( triggered() ), this, SLOT( configure() ) );
-   connect( exit_     , SIGNAL( triggered() ), this, SLOT( quit     () ) );
+   connect( sync_      , SIGNAL( toggled(bool) ), this, SLOT( sync      (bool) ) );
+   connect( numeration_, SIGNAL( toggled(bool) ), this, SLOT( numeration(bool) ) );
+   connect( save_      , SIGNAL( triggered() ), this, SLOT( save     () ) );
+   connect( configure_ , SIGNAL( triggered() ), this, SLOT( configure() ) );
+   connect( exit_      , SIGNAL( triggered() ), this, SLOT( quit     () ) );
 
    //------- operations -------
    connect( del_on_left_   , SIGNAL( triggered() ), this, SLOT( del_on_left   () ) );
@@ -365,8 +373,10 @@ void QBtMainWindow::retranslate()
    operations_->setTitle( tr( OPERATIONS ) );
    help_->setTitle( tr( HELP ) );
    //------- system -------
-   refresh_->setText( tr( REFRESH ) );
-   refresh_->setShortcut( tr( REFRESH_SKEY ) );
+   sync_->setText( tr( SYNC ) );
+   sync_->setShortcut( tr( SYNC_KEY ) );
+   numeration_->setText( tr( NUMERATION ) );
+   numeration_->setShortcut( tr( NUMERATION_KEY ) );
    save_->setText( tr( SAVE ) );
    save_->setShortcut( tr( SAVE_SKEY ) );
    configure_->setText( tr( CONFIGURE ) );
@@ -400,22 +410,30 @@ void QBtMainWindow::closeEvent( QCloseEvent* const in_event )
 //#                                                                 #
 //###################################################################
 
+//*******************************************************************
+// sync                                                 PRIVATE slot
+//*******************************************************************
+void QBtMainWindow::sync( const bool in_state )
+{
+   QBtEventsController::instance()->send_event( QBtEvent::SYNC, in_state );
+}
+// end of sync
 
 //*******************************************************************
-// refresh                                              PRIVATE slot
+// numeration                                           PRIVATE slot
 //*******************************************************************
-void QBtMainWindow::refresh()
+void QBtMainWindow::numeration( const bool in_state )
 {
-   QBtEventsController::instance()->send_event( QBtEvent::DIFF_CFG_CHANGED );
+   QBtEventsController::instance()->send_event( QBtEvent::NUMERATION, in_state );
 }
-// end of refresh
+// end of numeration
 
 //*******************************************************************
 // del_on_left                                          PRIVATE slot
 //*******************************************************************
 void QBtMainWindow::del_on_left()
 {
-   workspace_->del_on_left();
+   workspace_->del_on_A();
 }
 // end of del_on_left
 
@@ -424,7 +442,7 @@ void QBtMainWindow::del_on_left()
 //*******************************************************************
 void QBtMainWindow::del_on_right()
 {
-   workspace_->del_on_right();
+   workspace_->del_on_B();
 }
 // end of del_on_rigth
 
@@ -433,7 +451,7 @@ void QBtMainWindow::del_on_right()
 //*******************************************************************
 void QBtMainWindow::merge_to_left()
 {
-   workspace_->merge_to_left();
+   workspace_->merge_to_A();
 }
 // end of merge_to_left
 
@@ -442,7 +460,7 @@ void QBtMainWindow::merge_to_left()
 //*******************************************************************
 void QBtMainWindow::merge_to_right()
 {
-   workspace_->merge_to_right();
+   workspace_->merge_to_B();
 }
 // end of merge_to_rigth
 
@@ -484,8 +502,8 @@ void QBtMainWindow::about()
       "<font color=blue>A program for file comparing.</font>"
       "<hr>"
       "Author: Piotr Pszczolkowski<br>"
-      "Contact: piotr@beesoft.org<br>"
-      "Home page: www.beesoft.org"
+      "Contact: piotr@beesoft.at<br>"
+      "Home page: www.beesoft.at"
       "</qt>" );
    QMessageBox::about( this, QBtShared::program_name(), tr( text ) );
 }
